#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmath>
#include <vector>
#include <iostream>
#include <iomanip>
#include <stdexcept>

#include "meteo.h"


const std::string   sect_device("Devices");

Meteo::Meteo( Config& cfg, const std::string& dev ): port( dev ) {

    std::cerr << "Constructor Meteo" << std::endl;
}

Meteo::~Meteo(){
    std::cerr << "Destructor Meteo" << std::endl;
}

int Meteo::getData( MonitorInfo& c ) {
    std::string prefics;
    std::string in;
    in = port.read(200);
    if( in.find("WIND") !=  std::string::npos ) {
        std::istringstream str(in);
        str >> prefics >> c.wspeed_ins >> c.wdirec_ins >> c.rain;
//         std::cerr << "0:" << std::fixed << std::setprecision(1) << std::setw(6) << c.wspeed_ins
//         << std::setw(6) <<  c.wdirec_ins << std::setw(4) << c.rain << std::endl;
        return 0;
    }
    if( in.find("PTU") !=  std::string::npos )  {
        std::istringstream str(in);
        str >> prefics >> c.temper_ins >> c.temper_max >> c.temper_min >> c.rhumid_ins >> c.rhumid_max >> c.rhumid_min
                       >> c.dewpnt_ins >> c.dewpnt_max >> c.dewpnt_min >> c.press_ins >> c.press_max >> c.press_min ;
//         std::cerr << "1:" <<  std::fixed << std::setprecision(1)
//         << std::setw(6) << c.temper_ins << std::setw(6) << c.temper_max << std::setw(6) << c.temper_min
//         << std::setw(6) << c.rhumid_ins << std::setw(6) << c.rhumid_max << std::setw(6) << c.rhumid_min
//         << std::setw(6) << c.dewpnt_ins << std::setw(6) << c.dewpnt_max << std::setw(6) << c.dewpnt_min
//         << std::setw(6) << c.press_ins << std::setw(6) << c.press_max << std::setw(6) << c.press_min << std::endl;
    }
    return 1;
}

void Meteo::accumulate( std::vector<MonitorInfo>& v, MonitorInfo& acc ) {
    means( v, acc );
    mmaxs( v, acc );
    mmins( v, acc );
}

void Meteo::means( std::vector<MonitorInfo>& v, MonitorInfo& acc ) {
    unsigned int n = v.size();
    for( unsigned int i = 0; i < n; i++ ) {
        acc.wspeed_ins += v.at(i).wspeed_ins;
        acc.wdirec_ins += v.at(i).wdirec_ins;
    }
    acc.wspeed_ins /= n;
    acc.wdirec_ins /= n;
    acc.temper_ins = v.at(n-1).temper_ins;
    acc.rhumid_ins = v.at(n-1).rhumid_ins;
    acc.dewpnt_ins = v.at(n-1).dewpnt_ins;
    acc.press_ins  = v.at(n-1).press_ins;
}

void Meteo::mmaxs( std::vector<MonitorInfo>& v,  MonitorInfo& acc ) {
    unsigned int n = v.size();
    acc.wspeed_max = v.at(0).wspeed_ins;
    acc.wdirec_max = v.at(0).wdirec_ins;
    acc.rain       = v.at(0).rain;
    for( unsigned int i = 1; i < n; i++ ) {
        acc.wspeed_max = std::max( v.at(i).wspeed_ins, acc.wspeed_max );
        acc.wdirec_max = std::max( v.at(i).wdirec_ins, acc.wdirec_max );
        acc.rain       = std::max( v.at(i).rain, acc.rain ); 
    }
    acc.temper_max = v.at(n-1).temper_max;
    acc.rhumid_max = v.at(n-1).rhumid_max;
    acc.dewpnt_max = v.at(n-1).dewpnt_max;
    acc.press_max  = v.at(n-1).press_max;
}

void Meteo::mmins( std::vector<MonitorInfo>& v,  MonitorInfo& acc ) {
    unsigned int n = v.size();
    acc.wspeed_min = v.at(0).wspeed_ins;
    acc.wdirec_min = v.at(0).wdirec_ins;
//          std::cerr << "0:" << std::fixed << std::setprecision(1) << std::setw(6) << acc.wspeed_min
//          << std::setw(6) <<  acc.wdirec_min  << std::endl;
    for( unsigned int i = 1; i < n; i++ ) {
        acc.wspeed_min = std::min( v.at(i).wspeed_ins, acc.wspeed_min );
        acc.wdirec_min = std::min( v.at(i).wdirec_ins, acc.wdirec_min );
    }
    acc.temper_min = v.at(n-1).temper_min;
    acc.rhumid_min = v.at(n-1).rhumid_min;
    acc.dewpnt_min = v.at(n-1).dewpnt_min;
    acc.press_min  = v.at(n-1).press_min;
}

