#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include  <iostream>
#include <iomanip>
#include <map>
#include <sstream>
#include <string>

#include "defnames.h"
#include "astrotime.h"
#include "results.h"

AstroDateTime utc_out;

namespace {
    std::ostream& operator<<( std::ostream& stm, const std::ostringstream& str ){
        std::cout << str.str() << std::endl;
        stm << str.str() << std::endl;
        return stm;
    }
}

std::string Result::headstamp( const std::string& pref ) {
    utc_out.current(CONV);
    std::ostringstream t( pref + ' ' + utc_out.strDate() + ' ' + utc_out.strTime(0) + ' ' );
    return t.str();
}

Result::Result( Config& cfg ) {
    std::map<std::string,std::string> rPs;
    std::string str;
    bool append;
    std::string filename =  program_res + std::string(utc_out.strFileName())+extension_out;
    rfile.open( filename.c_str(), std::ios::in );
    if( ( append = rfile ) ){
        unsigned int i = 0;
        const unsigned int j = 22;          //    
        while( getline( rfile, str ) ){      //     P 
            if( str[0] != 'P' ) continue;
            if( (i =  str.find( "=", j )) != std::string::npos ) rPs[str.substr( j, i-j )] = str.substr(i);
            std::cout << str << std::endl;
        }
    }
    rfile.clear();
    if( append ) rfile.close();
    rfile.open( filename.c_str(), std::ios::out | std::ios::app );
    if( !rfile ) throw ErrFat( (ErrorsCode)errno, filename );
    rfile << comstamp() << progname << VERSION << " Compiled " << __TIME__ << " " << __DATE__<< std::endl;
    for( int i=0; i< (&cfg)->size(); i++ ) {
        unsigned int j = (&cfg)->Pstr(i).find( "=", 0 );
        if( rPs.find((&cfg)->Pstr(i).substr( 0, j )) == rPs.end() ||
            rPs[(&cfg)->Pstr(i).substr( 0, j )] != (&cfg)->Pstr(i).substr( j ) ) {
            std::ostringstream padd( parstamp() + (&cfg)->Pstr(i) );
            rfile << padd;
         }
    }
    rPs.clear();
//    std::cerr << "Constructor Result\n";
}

Result::~Result() {
//    std::cerr << "Destructor Result\n";
    utc_out.current(CONV);
    rfile <<  comstamp() << "Parked" << std::endl;
    rfile.close();
}

void Result::meteoAccResults( MonitorInfo& d ) {
    utc_out.current(CONV);
    std::ostringstream outstr;
    outstr << headstamp( "M" ) << std::fixed << std::setprecision(1)
           << std::setw(6) << d.temper_ins << std::setw(6) << d.temper_min << std::setw(6) << d.temper_max
           << std::setw(6) << d.wspeed_ins << std::setw(6) << d.wspeed_min << std::setw(6) << d.wspeed_max
           << std::setprecision(0)
           << std::setw(6) << d.wdirec_ins << std::setw(6) << d.wdirec_min << std::setw(6) << d.wdirec_max
           << std::setw(6) << d.rhumid_ins << std::setw(6) << d.rhumid_min << std::setw(6) << d.rhumid_max
           << std::setprecision(1)
           << std::setw(6) << d.dewpnt_ins << std::setw(6) << d.dewpnt_min << std::setw(6) << d.dewpnt_max
           << std::setw(7) << d.press_ins  << std::setw(7) <<  d.press_min << std::setw(7) << d.press_max
           << std::setw(4) << d.rain;
    rfile << outstr;
    data_copy = outstr.str().substr(2,outstr.str().size()-1);
}

double Result::tendency( double ins, double max,  double min) {
    return (max - ins > ins - min )? ins - max : ins - min;
}

void Result::meteoTrends( MonitorInfo& d ) {
    std::ostringstream outstr;
    outstr << std::fixed << std::setprecision(1)
           << std::setw(6) << tendency( d.temper_ins, d.temper_max, d.temper_min )
           << std::setw(6) << tendency( d.wspeed_ins, d.wspeed_max, d.wspeed_min ) << std::setprecision(0)
           << std::setw(6) << tendency( d.wdirec_ins, d.wdirec_max, d.wdirec_min )
           << std::setw(6) << tendency( d.rhumid_ins, d.rhumid_max, d.rhumid_min ) << std::setprecision(1)
           << std::setw(6) << tendency( d.dewpnt_ins, d.dewpnt_max, d.dewpnt_min )
           << std::setw(6) << tendency( d.press_ins,  d.press_max,  d.press_min  );
    trend_data = outstr.str();
}

void Result::meteoResults( MonitorInfo& d ) {
    utc_out.current(CONV);
    std::ostringstream outstr;
    outstr << headstamp( "m" ) << std::fixed << std::setprecision(1)
           << std::setw(6) << d.wspeed_ins << std::setw(6) << std::setprecision(0) << std::setw(6) << d.wdirec_ins
           << std::setw(6) << d.rain;
    rfile << outstr;
}

