#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <cmath>
#include <signal.h>
#include <vector>
#include <iostream>
#include <iomanip>
#include <stdexcept>
#include <unistd.h>

#include <astrotime.h>

#include "sens_device.h"
#include "results.h"

AstroDateTime utc;

const std::string   sect_device("Devices");
const std::string   subs_meteo("Conditions");

pthread_t Device::Data_thread;
int Device::proc_run = 0;

Device::Device( Config& cf,  bool tst ) try : device( cf.inp<std::string>( "General", "Common", "Device") ) {
    meteo_mast = std::auto_ptr<Meteo>( new Meteo( cf, device ) );
//    std::cerr << "Constructor Device" << std::endl;
}
catch ( const std::exception& e ) {
    ErrlogSingl::instance().wthrow( ERR_STD_EXCP, e.what() );
}

Device::~Device(void) {
}

void Device::monitor_run( double sp, double at ) {
    scan_period = sp;
    accum_time = at;
    accum_end = utc.currentSec() + accum_time;
    pthread_create( &Data_thread, NULL, &Device::thread, (void*)this );
}

void Device::monitor_stop(){
    if( pthread_cancel( Data_thread ) == ESRCH ) {
        ErrlogSingl::instance().write( "Data thread is terminated already");
        return;
    }
    if( pthread_join( Data_thread, NULL) != 0 ) {
        ErrlogSingl::instance().write(ERR_ALG, "Data thread pthread_join failed");
    }
}

void Device::monitor_result( int type, MonitorInfo& c ) {
//    out->meteoResults( c );
    accum.push_back( c );
    if( type ) {
        MonitorInfo reduced;
        meteo_mast->accumulate( accum, reduced );
        accum.clear();
        out->meteoAccResults( reduced );
        out->meteoTrends( reduced );
    }
}

bool Device::monitor_watch() {
    static int run_proc_prevs = -1;
    if( run_proc_prevs == proc_run ) return false;
    run_proc_prevs = proc_run;
    return true;
}

bool Device::expire = false;

void *Device::thread( void* a ) {
    Device* ptr = (Device*)a;
    MonitorInfo x;
    while( !expire ) {
        proc_run++;
        try {
//            if( sleep((int)ptr->scan_period) > 0 ) return 0;      //     scan_period
            if((ptr->meteo_mast)->connection()) {
                ptr->monitor_result( (ptr->meteo_mast)->getData( x ), x );
//                (ptr->meteo_mast)->getData( x );
            }
//            ptr->monitor_result( (ptr->meteo_mast)->getData( x ), x );
        }
        catch (const std::exception& e ) {
            ErrlogSingl::instance().write( ERR_EXC_STAT, "Monitor thread finished!" );
            return 0;
        }
    }
    ErrlogSingl::instance().write( "Monitor thread finished!" );
    return 0;
}



