#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <iomanip>
#include <vector>
#include <sstream>
#include <stdexcept>
#include <pthread.h>
//#include <signal.h>

#include <errlog.h>
#include <astrotime.h>

#include "sensors.h"
#include "defnames.h"

Sensors_CMD Sensors::cmd = { std::string(program_cfg), false, false, false };
Sensors* Sensors::self_ptr = 0;
const int Sensors::initLifeSec = 4;
const int Sensors::parkLifeSec = 4;
pthread_t Sensors::thread_id;
pthread_mutex_t Sensors::Lock::mtx = PTHREAD_MUTEX_INITIALIZER;

Sensors::Sensors(): Sensorscfg( cmd.cfgfile ), deviceSensors( Sensorscfg, cmd.test_mode ),
    meteoData( Sensorscfg ), meteoScan( Sensorscfg, deviceSensors, meteoData ) {
    the = &meteoScan;
//    std::cerr << "Constructor Sensors\n";
}

void Sensors::create() {
    Lock lock;
    ErrlogSingl::instance().reopen();
    if(self_ptr == 0) self_ptr = new Sensors();
}

void Sensors::destroy() {
    Lock lock;       // To prevent commands execution during Dome destruction
    if(self_ptr == 0) return;
    delete self_ptr;
    self_ptr = 0;
}

Sensors* Sensors::device_ptr() {
    Lock lock;
    if( self_ptr == 0 ) throw ErrPark(ERR_INI_OBJECT, "Sensors object");
    return self_ptr;
}

int Sensors::specify( const std::vector<std::string>& opts ) {
    throw ErrSyn( ERR_PROT_WRCOMM, "run" );
    return 1;
}

int  Sensors::stop( const std::vector<std::string>& opts ) {
    throw ErrSyn( ERR_PROT_WRCOMM, "stop" );
    return 1;
}

const int check_period = 30;

bool Sensors::works() {
    static AstroDateTime tmp;
    static double prevs = 0;
    if(  exist() && tmp.currentSec() - prevs > check_period ){
        if( !self_ptr->deviceSensors.monitor_watch() ) {
            ErrlogSingl::instance().write( "Monitor thread doesn't work!" );
            return false;
        }
        prevs = tmp.currentSec();
    }
    return true;
}


typedef void(Sensors::*SETF_T)(const std::string&);
typedef std::pair<const std::string, SETF_T> SET_PAIR;

void Sensors::set(const std::string& key ) {
    set(key, "-");
}
void Sensors::set(const std::string& key, const std::string& val) {
    throw ErrSyn( ERR_PROT_WRCOMM, "set" );
}

typedef std::string(Sensors::*GETF_T)();
typedef std::pair<const std::string, GETF_T> GET_PAIR;

std::string Sensors::get(const std::string& key) {
    static GET_PAIR get_kwd[] =  {
        GET_PAIR ("data",   &Sensors::data ),
        GET_PAIR ("wind",   &Sensors::get_wind ),
        GET_PAIR ("trend",  &Sensors::get_trend ),
        GET_PAIR ("rain",   &Sensors::get_rain )
    };
    static std::map<const std::string, GETF_T, Cmpi> getf(get_kwd, get_kwd + sizeof(get_kwd)/sizeof(get_kwd[0]));
    if( getf.find(key) == getf.end()) throw ErrSyn( ERR_PROT_WRKWD, key);
    return (this->*getf[key])();
}

std::string Sensors::data() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Meteo" );
    std::ostringstream str;
    str << "TEMP=" << it.at(2) << " T_MIN=" << it.at(3) << " T_MAX=" << it.at(4)
        << " WIND=" << it.at(5) << " WIND_DIR=" << it.at(8)
        << " RH=" << it.at(11) << " DEW_P=" << it.at(14) << " PRESS=" << it.at(17) << " RAIN=" << it.at(20) ;
    return str.str();
}

std::string Sensors::get_wind() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Meteo" );
    std::ostringstream str;
    str << "WIND=" << it.at(5) << " WIND_MIN=" << it.at(6)  << " WIND_MAX=" << it.at(7)
        << " WIND_DIR=" << it.at(8);
    return str.str();
}

std::string Sensors::get_trend() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.trend().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Meteo" );
    std::ostringstream str;
    str  << "TEMP=" << it.at(0) << " WIND=" << it.at(1) << " WIND_DIR=" << it.at(2)
         << " RH=" << it.at(3)  << " DUE_POINT=" << it.at(4) << " PRESSURE=" << it.at(5);
    return str.str();
}

std::string Sensors::get_rain() {
    std::vector<std::string> it;
    if( Tool::parse( meteoData.data().c_str(), it ) < 1 ) throw ErrFat( ERR_NOTHING_DONE, "Meteo" );
    std::string strain = (it.at(20) == "1") ? "rain" : "none"; 
    std::ostringstream str;
    str << "RAIN=" << strain;
    return str.str();
}

