#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <termios.h>
#include <sys/ioctl.h>
#include <errlog.h>
#include <stdexcept>

#include "serial.h"

const unsigned int spause = 40000;

Serial::Serial( const std::string& portname ) {
    fd = open( portname.c_str(), O_RDONLY|O_NOCTTY );
    if( fd < 0 ) ErrlogSingl::instance().wthrow( ERR_SERIAL, strerror(errno) );
//    fcntl(fd, F_SETFL, 0);
    tcgetattr( fd, &old_conf);
    serial_conf = old_conf;
    serial_conf.c_cflag &= ~CBAUD;
    serial_conf.c_cflag |= B9600;
    serial_conf.c_cflag &= ~CSIZE;
    serial_conf.c_cflag |= CS8;
    serial_conf.c_cflag &= ~CSTOPB;
    serial_conf.c_cflag &= ~(PARENB|PARODD);
    serial_conf.c_cflag |= CLOCAL|CREAD;
    serial_conf.c_iflag = IGNBRK;
    serial_conf.c_lflag |= ICANON;
    serial_conf.c_iflag &= ~(IXON|IXOFF|IXANY);
    serial_conf.c_oflag &= ~OPOST;
    serial_conf.c_cc[VMIN]  = 0;
    serial_conf.c_cc[VTIME] = 3;
    tcsetattr(fd, TCSANOW, &serial_conf );
    std::cerr << "Serial constructor" << std::endl;
}

Serial::~Serial() {
    tcsetattr(fd, TCSANOW, &old_conf );
    close( fd );
    std::cerr << "Serial destructor" << std::endl;
}

size_t Serial::send( std::string& s ) {
    if( s.empty() ) return 0;
    ::ioctl(fd, TCFLSH, 2);
    ssize_t res = ::write( fd, s.data(), s.size() );
    if( res < 0 ) throw std::runtime_error( strerror(errno) );
    last_sent = s;
    usleep( spause );
    return res;
}

std::string Serial::ask( std::string& s, size_t n ) {
    size_t size = 0;
    ssize_t res = 0;
    size_t att = n;
    send( s );
    while( size < n && att-- ) {
        res = ::read( fd, inp_buff+size, n );
        if ( res < 0 ) throw std::runtime_error( strerror(errno) );
        size += res;
    }
    inp_buff[size] = 0;
    usleep( spause );
    return last_repl = ( size > 0 )? std::string(inp_buff) : std::string();
}

std::string Serial::read( size_t n ) {
    size_t size = 0;
    ssize_t res = 0;
//    size_t att = n;
//    while( size < n && att-- ) {
        res = ::read( fd, inp_buff+size, n );
        if ( res < 0 ) throw std::runtime_error( strerror(errno) );
        size += res;
//    }
    if( size > 0 ) inp_buff[size-2] = 0;
    return last_repl = ( size > 0 )? std::string(inp_buff) : std::string();
}


std::string Serial::ask( std::string& s, size_t n, int time_in_s ) {
    size_t size = 0;
    ssize_t res = 0;
    size_t att = time_in_s;
    send( s );
    while( size < n && att-- ) {
        res = ::read( fd, inp_buff+size, n );
        if ( res < 0 ) throw std::runtime_error( strerror(errno) );
        size += res;
    }
    inp_buff[size] = 0;
    usleep( spause );
    return last_repl = ( size > 0 )? std::string(inp_buff) : std::string();
}

std::string Serial::ask( std::string& s, char lim ) {
    size_t size = 0;
    ssize_t res = 0;
    size_t att = 10;
    send( s );
    while( ( size == 0 || *(inp_buff+size-1) != lim ) && att-- ) {
        res = ::read( fd, inp_buff+size, 128 );
        if ( res < 0 ) throw std::runtime_error( strerror(errno) );
        size += res;
    }
    if( size != 0 && inp_buff[size-1] == lim ) size -= 1;
    inp_buff[size] = 0;
    usleep( spause );
    return last_repl = ( size > 0 )? std::string(inp_buff) : std::string();
}

void Serial::up_baude_rate() {
    serial_conf.c_cflag &= ~CBAUD;
    serial_conf.c_cflag |= B57600;
    tcsetattr(fd, TCSANOW, &serial_conf );
}

void Serial::down_baude_rate() {
    serial_conf.c_cflag &= ~CBAUD;
    serial_conf.c_cflag |= B9600;
    tcsetattr(fd, TCSANOW, &serial_conf );
}
